/*
  Program:  servo player7.c

  Status:   Good  - NOTE: Specify Servo Command file and Profile in code below!

  Author:   Brian Lincoln (lightman)
  Date:     6/28/14

  Description:
      1) To playback and control multiple servos by reading commands from SD card and outputting at ~20 Hz rate
      2) Uses pre determined servo file name and Profile file name
      3) Trigger show with external signal (0 = wait  1 = Start Show); Cannot stop show in middle- Plays through


  Revision Updates:
  7/4/14  BTL - Original from servo record playback34.c
  7/5/14  BTL - Cleaned up text notifications and code; Removed i sample record counter
  8/21/14 BTL - Updated with const char string names
              - Added Pin 0 as Start Show flag
  8/22/14 BTL - Added Rev pot to profile code; Updated Print statements
              - Corrected fp open for profile - removed quotes from name



  Propeller I/O:

  Pin   Description
  0     Start Show flag (0 = idle/wait   1 = Start Show)
  1     Spare
  2     Spare
  3     Spare
  4     Spare
  5     Spare
  6     Spare
  7     Spare
  8     Spare
  9     Spare 
  10    Servo Motor8 
  11    Servo Motor7
  12    Servo Motor6
  13    Servo Motor5
  14    Servo Motor4
  15    Servo Motor3
  16    Servo Motor2
  17    Servo Motor1
  18 to 31 are reserved for SD card, Audio, DAC, LEDs, etc


*/

#include "simpletools.h"                      // Include simpletools header
#include "servo.h"


//--------- Names of Servo cmd file and Profile ------------------------------------------------------

const char servoname[] = {"shead2"};          // <==================== CHANGE SERVO FILE NAME HERE
const char profilename[] = {"phead"};         // <==================== CHANGE PROFILE FILE NAME HERE


//----------- Variable / Constants Declarations -----------------------------------------------------

int n, status, mode, tmp, s;                  // Number & status variables & mode, Servo number

int mid = 900;                                // Midpoint of servos for new show initialization


// ------ Set Default Profile Settings and Iniitalze Arrays -----------------------------------------
int servocmd[] =  {0,  900,  900,  900,  900,  900,  900,  900,  900}; // Est. array & Set Initial Cond for 8 servo angles
int servolow[] =  {0,   50,   50,   50,   50,   50,   50,   50,   50}; // Est. array & Set Lower limit defaults for 8 servos
int servohigh[] = {0, 1750, 1750, 1750, 1750, 1750, 1750, 1750, 1750}; // Est. array & Set upper limit defaults for 8 servos
int servorest[] = {0,  900,  900,  900,  900,  900,  900,  900,  900}; // Est. array & Set Rest Positions for 8 servo angles
int potrev[] =    {0,    0,    0,    0,    0,    0,    0,    0,    0}; // Est. Array & set reversal flag (0 = normal,  1 = reverse)

int DO = 22, CLK = 23, DI = 24, CS = 25;      // SD card pins on Propeller BOE



//------------- MAIN FUNCTION ----------------------------------------------------------------
int main(void)                                // main function
{
  sd_mount(DO, CLK, DI, CS);                  // Mount SD card

  FILE* fp;                                   // Open a pointer for SD card Servo Files
  FILE* fpprof;                               // Open another pointer SD card for Profile files

//------------- Initialize -------------------------------------------------------------------
  for(int i = 1; i <= 8; i++)                 // Counter for 8 servos
  {
    servo_angle((18-i), mid);                 // Initialize all Servos to midpoint
  } //end for



//----------- Print out Banner Header --------------------------------------------------------
  print("----- Servo Player ------\n");
  print("       Version 7\n\n");

  print("Uses pre-defined file names in code\n");

  //-------- Enter Profile File Names & Load file -----------------------------------------------------
  fpprof = fopen(profilename, "r");           // Open Profile file for read

  if(fpprof)                                  // if file exists (fpprof = 1) then proceed to load
  {
    for(int i = 1; i <= 8; i++)               // Readin Profile
    {
      fread(&servolow[i], 4, 1, fpprof);      // read profile from SD card New File
      fread(&servohigh[i], 4, 1, fpprof);
      fread(&servorest[i], 4, 1, fpprof);
      fread(&potrev[i], 4, 1, fpprof);
    } //end for
    print("Profile Loaded Successfully\n\n");
  } // end if
  else
  {
    print("Profile not found...Using Default Profile\n\n");
  }
  fclose(fpprof);                             // Close the files



  //---------- Main Processing Loop ---------------------------------------------------------------
  while(1)                                    // Reapeat main loop forever
  {   
      //-------- Set Servos to rest position ------------------------------------------------------
      print("Servos at Rest - Wait for Trigger\n");
      for(int i = 1; i <= 8; i++)                   // Counter for 8 servos
      {
        servo_angle((18-i), servorest[i]);          // Initialize all Servos to rest position
      } //end for


      //----- Check for Start of Show --------------------------------------------------------------
      if (input(0) == 1)                            // Check Pin 0 for ext. Trigger; If high (1) then execute servo routine
      {
        //----------------  Playback mode -----------------------------------------------------------
        print("\nTrigger Received - Start Show\n"); 

        fp = fopen(servoname, "r");                 // Open Servo file for reading

        print("\nStart Playback....\n");

        while(1)                                    // Run servo routine until no more data in file
        {
          for(int j = 1; j <= 8; j++)               // Read all 8 Servo cmd data - Store in array
          {
            status = fread(&servocmd[j], 4, 1, fp);   
            if(servocmd[j] < servolow[j]) servocmd[j] = servolow[j];    // Limit data to profile
            if(servocmd[j] > servohigh[j]) servocmd[j] = servohigh[j];     
          } // end for

          if (!status) break;                       // If end of file, break

          for(int j = 1; j <= 8; j++)               // 8 servo commands to move servos
          {
            servo_angle((18-j), servocmd[j]);       // Move all 8 Servos 
          }  //end for

          pause(45);                                // Wait 45mS - trimmed for 20 HZ with Print statement

        } //end while 

        fclose(fp);                                 // Routine done so Close the file
        print("Done Playback\n\n");

      } //end if

   pause(100);                                      // Wait a little before returning from loop

  }  // End While
}    

